@@ -1,12 +1,23 @@
-import attr
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Generic, TypeVar
+
+import attrs
 
 import trio
-from trio._util import Final
+from trio._util import final
 
-from .abc import HalfCloseableStream
+from .abc import AsyncResource, HalfCloseableStream, ReceiveStream, SendStream
+
+if TYPE_CHECKING:
+    from typing_extensions import TypeGuard
 
 
-async def aclose_forcefully(resource):
+SendStreamT = TypeVar("SendStreamT", bound=SendStream)
+ReceiveStreamT = TypeVar("ReceiveStreamT", bound=ReceiveStream)
+
+
+async def aclose_forcefully(resource: AsyncResource) -> None:
     """Close an async resource or async generator immediately, without
     blocking to do any graceful cleanup.
 
@@ -36,8 +47,17 @@ async def aclose_forcefully(resource):
         await resource.aclose()
 
 
-@attr.s(eq=False, hash=False)
-class StapledStream(HalfCloseableStream, metaclass=Final):
+def _is_halfclosable(stream: SendStream) -> TypeGuard[HalfCloseableStream]:
+    """Check if the stream has a send_eof() method."""
+    return hasattr(stream, "send_eof")
+
+
+@final
+@attrs.define(eq=False, hash=False, slots=False)
+class StapledStream(
+    HalfCloseableStream,
+    Generic[SendStreamT, ReceiveStreamT],
+):
     """This class `staples <https://en.wikipedia.org/wiki/Staple_(fastener)>`__
     together two unidirectional streams to make single bidirectional stream.
 
@@ -72,34 +92,36 @@ class StapledStream(HalfCloseableStream, metaclass=Final):
 
     """
 
-    send_stream = attr.ib()
-    receive_stream = attr.ib()
+    send_stream: SendStreamT
+    receive_stream: ReceiveStreamT
 
-    async def send_all(self, data):
+    async def send_all(self, data: bytes | bytearray | memoryview) -> None:
         """Calls ``self.send_stream.send_all``."""
         return await self.send_stream.send_all(data)
 
-    async def wait_send_all_might_not_block(self):
+    async def wait_send_all_might_not_block(self) -> None:
         """Calls ``self.send_stream.wait_send_all_might_not_block``."""
         return await self.send_stream.wait_send_all_might_not_block()
 
-    async def send_eof(self):
+    async def send_eof(self) -> None:
         """Shuts down the send side of the stream.
 
-        If ``self.send_stream.send_eof`` exists, then calls it. Otherwise,
-        calls ``self.send_stream.aclose()``.
-
+        If :meth:`self.send_stream.send_eof() <trio.abc.HalfCloseableStream.send_eof>` exists,
+        then this calls it. Otherwise, this calls
+        :meth:`self.send_stream.aclose() <trio.abc.AsyncResource.aclose>`.
         """
-        if hasattr(self.send_stream, "send_eof"):
-            return await self.send_stream.send_eof()
+        stream = self.send_stream
+        if _is_halfclosable(stream):
+            return await stream.send_eof()
         else:
-            return await self.send_stream.aclose()
+            return await stream.aclose()
 
-    async def receive_some(self, max_bytes=None):
+    # we intentionally accept more types from the caller than we support returning
+    async def receive_some(self, max_bytes: int | None = None) -> bytes:
         """Calls ``self.receive_stream.receive_some``."""
         return await self.receive_stream.receive_some(max_bytes)
 
-    async def aclose(self):
+    async def aclose(self) -> None:
         """Calls ``aclose`` on both underlying streams."""
         try:
             await self.send_stream.aclose()
