@@ -375,7 +375,7 @@ def _sniff_sockopts_for_fileno(
     # and then we'll throw it away and construct a new one with the correct metadata.
     if sys.platform != "linux":
         return family, type_, proto
-    from socket import (  # type: ignore[attr-defined]
+    from socket import (  # type: ignore[attr-defined,unused-ignore]
         SO_DOMAIN,
         SO_PROTOCOL,
         SO_TYPE,
